#pragma once
#include <string>
#include <vector>

namespace JJX {
    class ParsedValue;
    class ParsedObject;
    class ParsedString;
    class ParsedNumber;
    class ParsedArray;
    class ParsedBool;
    class ParsedNULL;
    class ParsedFile;
}

/** This is kind-of slow, But it makes it easy to traverse the file using dynamic_cast.
 * If you need bleeding edge speed while parsing you're not doing it right anyway :shrug: - Redacted */
class JJX::ParsedValue {
protected:
    std::string key;
public:
    [[nodiscard]] std::string GetKey() const;
public:
    virtual ~ParsedValue() = default;
};

class JJX::ParsedObject : public ParsedValue {
private:
    std::vector<ParsedValue*> values;
protected:
    explicit ParsedObject(const std::vector<ParsedValue*>& values);
public:
    void EraseChildren();
     [[nodiscard]] std::vector<ParsedValue*> GetValue() const;
public:
    static ParsedObject* Create(const std::vector<ParsedValue*>& values);

};

class JJX::ParsedString : public ParsedValue {
private:
    std::string value;
protected:
    ParsedString(const std::string& key, const std::string& value);
public:
    [[nodiscard]] std::string GetValue() const;
public:
    /** This is a trick borrowed from Java, But in this instance, it's to ensure that they're heap allocated. */
    static ParsedString* Create(const std::string& key, const std::string& value);
};

class JJX::ParsedNumber : public ParsedValue {
private:
    double value;
protected:
    ParsedNumber(const std::string& key, const std::string& value);
    ParsedNumber(const std::string& key, double value);
public:
    [[nodiscard]] double GetValue() const;
public:
    static ParsedNumber* Create(const std::string& key, const std::string& value);
    static ParsedNumber* Create(const std::string& key, double value);
};

class JJX::ParsedArray : public ParsedValue {
private:
    std::vector<ParsedValue*> values;
protected:
    ParsedArray(const std::string& key, const std::vector<ParsedValue*>& values);
public:
    void EraseChildren();
    [[nodiscard]] std::vector<ParsedValue*> GetValue() const;
public:
    static ParsedArray* Create(const std::string& key, const std::vector<ParsedValue*>& values);
};

class JJX::ParsedBool : public ParsedValue {
private:
    bool value;
protected:
    ParsedBool(const std::string& key, bool value);
public:
    [[nodiscard]] bool GetValue() const;
public:
    static ParsedBool* Create(const std::string& key, bool value);
};

class JJX::ParsedNULL : public ParsedValue {
protected:
    explicit ParsedNULL(const std::string& key);
public:
    [[nodiscard]] nullptr_t GetValue() const;
public:
    static ParsedNULL* Create(const std::string& key);
};

class JJX::ParsedFile {
private:
    std::vector<ParsedValue*> content;
public:
    std::vector<ParsedValue*> GetContent();
public:
    explicit ParsedFile(const std::vector<ParsedValue*>& content);
    ~ParsedFile();

};

