#include <JJX/ParsedFile.h>

using namespace JJX;

std::string JJX::ParsedValue::GetKey() const {
    return key;
}

JJX::ParsedString::ParsedString(const std::string &key, const std::string& value) {
    this->key = key;
    this->value = value;
}

std::string JJX::ParsedString::GetValue() const {
    return value;
}

ParsedString* ParsedString::Create(const std::string& key, const std::string& value) {
    return new ParsedString(key, value);
}

double JJX::ParsedNumber::GetValue() const {
    return value;
}

JJX::ParsedNumber::ParsedNumber(const std::string& key, const std::string& value) {
    this->key = key;
    this->value = std::stod(value);
}

JJX::ParsedNumber::ParsedNumber(const std::string& key, double value) {
    this->key = key;
    this->value = value;
}

ParsedNumber* ParsedNumber::Create(const std::string& key, const std::string& value) {
    return new ParsedNumber(key, value);
}

ParsedNumber* ParsedNumber::Create(const std::string& key, double value) {
    return new ParsedNumber(key, value);
}

std::vector<ParsedValue*> JJX::ParsedArray::GetValue() const {
    return values;
}

ParsedArray::ParsedArray(const std::string& key, const std::vector<ParsedValue*>& values) {
    this->key = key;
    this->values = values;
}

void ParsedArray::EraseChildren() {
    for (auto& value : values)
        if (auto* array = dynamic_cast<ParsedArray*>(value))
            array->EraseChildren();
        else if (auto* object = dynamic_cast<ParsedObject*>(value))
            object->EraseChildren();
        else
            delete value;
    values = {};
}

ParsedArray* ParsedArray::Create(const std::string& key, const std::vector<ParsedValue*>& values) {
    return new ParsedArray(key, values);
}

ParsedBool::ParsedBool(const std::string& key, bool value) {
    this->key = key;
    this->value = value;
}

bool ParsedBool::GetValue() const {
    return value;
}

ParsedBool* ParsedBool::Create(const std::string& key, bool value) {
    return new ParsedBool(key, value);
}

nullptr_t ParsedNULL::GetValue() const {
    return nullptr;
}

ParsedNULL::ParsedNULL(const std::string& key) {
    this->key = key;
}

ParsedNULL* ParsedNULL::Create(const std::string& key) {
    return new ParsedNULL(key);
}

std::vector<ParsedValue*> ParsedFile::GetContent() {
    return content;
}

// kick off the recursive deletion.
ParsedFile::~ParsedFile() {
    for (auto& value : content)
        if (auto* array = dynamic_cast<ParsedArray*>(value))
            array->EraseChildren();
        else if (auto* object = dynamic_cast<ParsedObject*>(value))
            object->EraseChildren();
        else
            delete value;
        content = {};
}

ParsedFile::ParsedFile(const std::vector<ParsedValue*>& content) {
    this->content = content;
}

ParsedObject::ParsedObject(const std::vector<ParsedValue*>& values) {
    this->key = "";
    this->values = values;
}

ParsedObject* ParsedObject::Create(const std::vector<ParsedValue*>& values) {
    return new ParsedObject(values);
}

void ParsedObject::EraseChildren() {
    for (auto& value : values)
        if (auto* array = dynamic_cast<ParsedArray*>(value))
            array->EraseChildren();
        else if (auto* object = dynamic_cast<ParsedObject*>(value))
            object->EraseChildren();
        else
            delete value;
    values = {};

}

std::vector<ParsedValue*> ParsedObject::GetValue() const {
    return values;
}
