#include <lexer.hpp>
#include <json.hpp>
#include <tuple>
#include <vector>
#include <error_handling.hpp>

namespace json {
	int lexers::lex_whitespace(std::string raw_json, int index)
	{
		while (std::isspace(raw_json[index])) {
			if (index == raw_json.length()) {
				break;
			}
			index++;
		}
		return index;
	}

	std::tuple<json::token, int, std::string> lexers::lex_syntax(std::string raw_json, int index)
	{

		json::token token{"", token_type::syntax, index};
		std::string value = "";
		auto c = raw_json[index];
		if (c == '[' || c == ']' || c == '{' || c == '}' || c == ':' || c == ',') {
			token.value += c;
			index++;
		}

		return {token, index, ""};
	}

	std::tuple<json::token, int, std::string> lexers::lex_string(std::string raw_json, int original_index)
	{
		int index = original_index;
		json::token token {"", token_type::string, index};
		std::string value = "";
		auto c = raw_json[index];
		if (c != '"') {
			return {token, original_index, ""};
		}
		index++;

		// TODO: handle nested quotes
		while (c = raw_json[index], c != '"') {
			if (index == raw_json.length()) {
				//return {token, index, format_error("Unexpected EOF while lexing string", raw_json, index)};
                throw Exception("Unexpected EOF while lexing string", raw_json, index);
			}

			token.value += c;
			index++;
		}
		index++;

		return {token, index, ""};
	}

	std::tuple<json::token, int, std::string> lexers::lex_number(std::string raw_json, int original_index)
	{
		int index = original_index;
		json::token token {"", token_type::number, index};
		std::string value = "";
		bool decimal_present = false;

		// Negative numbers.
		if (raw_json[index] == '-') {
			token.value += raw_json[index];
			index++;
		}

		while(true) {
			if (index == raw_json.length())
				break;

			auto c = raw_json[index];
			if (c == '.') {
				if (decimal_present)
					break;
				decimal_present = true;
				//token.value += c; // This caused two dots to be inserted.
			}

			// Scientific notation.
			else if (c == 'E' || c == 'e') {
				token.value += c;
				index++;
				if (raw_json[index] == '-') {
					token.value += raw_json[index];
					index++;
				}
				continue; // Loop early.
			}

			// Only regex non-numeric values if we didn't catch it earlier.
			else if (!(c >= '0' && c <= '9'))
				break;

			token.value += c;
			index++;
		}
		return {token, index, ""};
	}

	std::tuple<json::token, int, std::string> lexers::lex_keyword(std::string raw_json, std::string keyword,
		json::token_type type, int original_index)
	{
		int index = original_index;
		json::token token{"", type, index};

		while (keyword[index - original_index] == raw_json[index]) {
			if (index == raw_json.length()) {
				break;
			}
			index++;
		}
		if (index - original_index == keyword.length()) {
			token.value = keyword;
		}
		return {token, index, ""};
	}

	std::tuple<json::token, int, std::string> lexers::lex_null(std::string raw_json, int index)
	{
		return lex_keyword(raw_json, "null", token_type::null, index);
	}

	std::tuple<json::token, int, std::string> lexers::lex_true(std::string raw_json, int index)
	{
		return lex_keyword(raw_json, "true", token_type::boolean, index);
	}

	std::tuple<json::token, int, std::string> lexers::lex_false(std::string raw_json, int index)
	{
		return lex_keyword(raw_json, "false", token_type::boolean, index);
	}
}
